/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2000-2003 Justin Kirby
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 */
#ifndef OAPR_EXC_BASE_HPP
#define OAPR_EXC_BASE_HPP
#include "oapr/oapr_buildconfig.hpp"
#include <string>
#include <list>
#include <exception>
#include "oapr_exc_utils.hpp"

FILE_VERSION("$Revision: 1.4 $",oapr_exc_base_hpp);

namespace oapr
{
	class LIBOAPR_API std::exception;
  class LIBOAPR_API exception_base : public std::exception
  {
  public:
    enum Level{
      EXC_Emerg=0,
      EXC_Alert,
      EXC_Crit,
      EXC_Error,
      EXC_Warning,
      EXC_Notice,
      EXC_Info,
      EXC_Debug};
    typedef std::list<exception_base> exc_cont;
  public:
    exception_base(int line,
                   std::string method,
                   std::string file,
                   std::string msg,
                   int level = 0)
    : m_line(line),
      m_method(method),
      m_file(file),
      m_source(namespace_from_pretty_function(method)),
      m_message(msg),
      m_level(level)
    {
    }
    exception_base(const exception_base& bs,
                   int line,
                   std::string method,
                   std::string file,
                   std::string src,
                   std::string msg,
                   int level = 0)
    : m_line(line),
      m_method(method),
      m_file(file),
      m_source(src),
      m_message(msg),
      m_level(level),
      m_stack(bs.m_stack)
    {
      m_stack.push_back(bs);
    }
    exception_base(const exception_base& bs)
      : m_line(bs.m_line),
        m_method(bs.m_method),
        m_file(bs.m_file),
        m_source(bs.m_source),
        m_message(bs.m_message),
        m_level(bs.m_level),
        m_stack(bs.m_stack)
    {}
    virtual ~exception_base() throw() {}
    virtual const int line() const throw() {
      return m_line;
    }
    virtual const char* what() const throw() {
      return m_message.c_str();
    }
    virtual const char* method() const throw() {
      return m_method.c_str();
    }
    virtual const char* file() const throw() {
      return m_file.c_str();
    }
    virtual const char* source() const throw() {
      return m_source.c_str();
    }
    virtual const int level() const throw() {
      return m_level;
    }
    /* iterator accessor functions */
    exc_cont::const_iterator begin() const {
      return m_stack.begin();
    }
    exc_cont::const_iterator end() const {
      return m_stack.end();
    }

  protected:
    void message(const char *msg) { m_message.assign(msg); }
  private:
    int m_line;
    std::string m_method;
    std::string m_file;
    std::string m_source;
    std::string m_message;
    int m_level;
    exc_cont m_stack;
  };

};

#endif
