/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2003 Matt Carter. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */
#include "oapr/oapr.hpp"
#include "oapr/oapr_buildconfig.hpp"
#include "oapr/oapr_getopt.hpp"
#include "oapr/oapr_exception.hpp"
#include <string>
#include <iostream>

FILE_VERSION("$Revision: 1.12.2.1 $",oapr_getopt_cpp);

using namespace std;
using namespace oapr;

cmdline_processor::cmdline_processor(const pool::pool_ptr& p,
				     int argc, const char * const *argv,
                                     const static_opts arg_definition[],
				      bool allow_interleave,
                                     bool throw_on_bad_args)
  : m_opts(arg_definition),
    m_state(APR_SUCCESS),
    m_opt(0),
    m_arg(NULL),
    m_throw(throw_on_bad_args),
    m_pool(p)
{
  OAPR_CHECK(apr_getopt_init(&m_os, p->getp(), argc, argv));
  m_os->interleave = allow_interleave;
  m_os->errfn = NULL;
}

bool cmdline_processor::getopt()
{
  apr_status_t rc = apr_getopt_long(m_os, m_opts, &m_opt, &m_arg);
  if(rc == APR_SUCCESS) return true;
  if(rc == APR_EOF) return false;
  if(m_throw) OAPR_THROW_APR(rc);
  m_state = rc; return true;
}

void oapr::usage(const static_opts args[], const char *cmd)
{
  if(!cmd) OAPR_THROW(runtime_error, "No application name set");
  ostream &text = cerr;
  text << "Usage: " << cmd;
  if(args->optch != 0)
  {
    text << " " << "[options]" << endl
         << "Options:" << endl;
    const static_opts *arg = args;
    while(arg->optch)
    {
      text << "  -" << (char) arg->optch;
      if(arg->name) text << ", --" << arg->name;
      if(arg->description) text << "  \t" << arg->description;
      text << endl;
      arg++;
    }
  }
  else
  {
    text << endl;
  }
}
