/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2000-2003 Justin Kirby
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 */
#include "oapr/oapr_buildconfig.hpp"
#include "oapr/oapr_io_pipe.hpp"
#include "oapr/oapr_exception.hpp"
#include "oapr/oapr_exc_utils.hpp"

FILE_VERSION("$Revision: 1.5.2.1 $",oapr_io_pipe_cpp);
namespace oapr
{
  apr_status_t basic_pipe::open(apr_int32_t flags, apr_int32_t perms)
  {
    if(is_open()){
      OAPR_THROW(logic_error,"pipe is already open");
    }
    apr_status_t rv;
    rv=apr_file_pipe_create(&m_readp,&m_writep,m_pool->getp());
    if(rv==APR_SUCCESS){
      m_type=pipe;
      m_isOpen=true;
    }
    return rv;
  }
  apr_status_t basic_pipe::open(const std::string &name,
			apr_int32_t flags,
			apr_int32_t perms)
  {
    if(is_open()){
      OAPR_THROW(logic_error,"pipe is already opened");
    }
    apr_status_t rv;
    if(flags&APR_CREATE){
      OAPR_CHECK(apr_file_namedpipe_create(name.c_str(),perms,m_pool->getp()));
    }

    rv=apr_file_open(&m_file,name.c_str(),flags,perms,m_pool->getp());
    if(rv==APR_SUCCESS){
      m_type=named;
      m_isOpen=true;
    }
    return rv;
  }
  apr_status_t basic_pipe::close()
  {
    if(m_type==pipe){
      OAPR_CHECK(apr_file_close(m_readp));
      OAPR_CHECK(apr_file_close(m_writep));
      return APR_SUCCESS;
    }else{
      return basic_file::close();
    }
  }
  apr_size_t basic_pipe::write(const void* buff,apr_size_t toWrite,bool full)
  {
    if(m_type==pipe){
      apr_size_t rv(toWrite);
      OAPR_CHECK(apr_file_write(m_writep,buff,&rv));
      return rv;
    }else{
      return basic_file::write(buff,toWrite,full);
    }
  }
  apr_size_t basic_pipe::read( void* buff,apr_size_t toRead,bool full)const
  {
    if(m_type==pipe){
      apr_size_t rv(toRead);
      OAPR_CHECK(apr_file_read(m_readp,buff,&rv));
      return rv;
    }else{
      return basic_file::read(buff,toRead,full);
    }
  }

  apr_status_t basic_pipe::flush()
  {
    if(m_type==pipe){
      return apr_file_flush(m_writep);
    }else{
      return basic_file::flush();
    }
  }
  void basic_pipe::timeout(apr_interval_time_t to)
  {
    m_timeout=to;
    if(m_type==pipe){
      OAPR_CHECK(apr_file_pipe_timeout_set(m_readp,m_timeout));
      OAPR_CHECK(apr_file_pipe_timeout_set(m_writep,m_timeout));
    }else{
      OAPR_CHECK(apr_file_pipe_timeout_set(m_file,m_timeout));
    }
  }
};
