/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2000-2003 Justin Kirby
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 */
#ifndef OAPR_IO_SOCKETINFO_HPP
#define OAPR_IO_SOCKETINFO_HPP
#include <string>
#include "oapr/oapr.hpp"
#include "apr_network_io.h"
#include "oapr/oapr_pool_mgr.hpp"
#include "boost/shared_ptr.hpp"

namespace oapr
{
  class LIBOAPR_API sockinfo
  {
  public:
    typedef pool_mgr::pool_t pool_t;
    typedef pool_mgr::pool_ptr pool_ptr;
  public:

    /* need to deal with the ctor issue
     * ways to ctor
     * - plain, i.e. init vars later
     * - from, hostname, family, port, flags
     */
    sockinfo(const pool_ptr& p)
      : m_pool(p)
	,m_addrt(0)
    {
    }
    sockinfo(const pool_ptr& p, const std::string &host, apr_int32_t fam,
	     apr_port_t port, apr_int32_t flags)
      : m_pool(p),
	m_addrt(0)
    {
      create(host,fam,port,flags);
    }
    sockinfo(const sockinfo& si)
      : m_pool(si.m_pool),
	m_addrt(si.m_addrt)
    {
    }
    ~sockinfo()
    {
    }

    /* uses:
     * - bind
     * - connect
     * - sendto & recv from
     */
    void create(const std::string& hostname,
		apr_int32_t family,
		apr_port_t port,
		apr_int32_t flags);

    apr_sockaddr_t* sockaddr()const{
      return m_addrt;
    }
    
    /* features we need
     * - hostname from current addr
     * - hostname from machine
     * - service name, either port of service name for the port
     * - port
     * - family
     * - ip addr (also need multifunction ipaddr v4/v6)
     * - comparison function
     * - set port by service name
     * - ipsubnet create <-- why?
     * - ipsubnet test
     */
    /**
     * This will return the hostname in one of two ways.
     * If the sockaddr_t is not set, it will get this machines hostname.
     * else it will use the internal sockaddr_t.
     * @return returns the hostname
     */
    std::string hostname()const;
    /**
     * Retrieve either the service name of the port
     * or the port in string format.
     * @return servicename
     */
    std::string service()const;
    /**
     * Get the port of the socket addr
     * @return the port
     */
    apr_port_t port()const;

    apr_status_t port(apr_port_t p);


    /**
     * Get the family of the socket
     *
     */
    /**\todo need to figure out how to get this info*/
    int family()const;

    /**
     * get the ip address of the socket, supports both v4 and v6
     * @return the ip address
     */
    std::string ip()const;


    /* issues need resolving:
     * - deal with apr_parse_addr_port
     * - deal with apr_socket_data_get/set
     */

  private:
    pool_ptr m_pool;    
    apr_sockaddr_t *m_addrt;
  };
};
#endif
