/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2000-2003 Justin Kirby
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 */
#ifndef OAPR_OFILESTREAM_HPP
#define OAPR_OFILESTREAM_HPP
#include "oapr/oapr.hpp"
#include "oapr/oapr_buildconfig.hpp"
#include <ostream>
#include "boost/scoped_ptr.hpp"
#include "oapr/oapr_exception.hpp"
#include "oapr/oapr_exc_utils.hpp"
#include "oapr/oapr_filebuf.hpp"
FILE_VERSION("$Revision: 1.5 $",oapr_ofilestream_hpp);

namespace oapr
{
  template<typename charT,typename traits, std::streamsize bufszT=255>
  class basic_ofilestream : public std::basic_ostream<charT,traits>
  {
  public:
    //standard typedefs
    typedef charT                     char_type;
    typedef typename traits::int_type int_type;
    typedef typename traits::pos_type pos_type;
    typedef typename traits::off_type off_type;
    typedef traits                    traits_type;

    //non standard typdefs
    //typedef put stream buf type here
    typedef std::basic_ios<char_type,traits>           ios_type;
    typedef std::basic_ostream<char_type,traits>       ostream_type;
    typedef std::ostreambuf_iterator<char_type,traits> ostreambufiter_type;
    typedef basic_filebuf<char_type,traits_type>       filebuf_type;
    typedef basic_ofilestream<char_type,traits_type>   ofilestream_type;

    typedef pool_mgr::pool_t pool_t;
    typedef pool_mgr::pool_ptr pool_ptr;

  private:
    boost::scoped_ptr<filebuf_type> m_filebuf;

  public:
    basic_ofilestream(const pool_ptr& p)
      : ostream_type(NULL)
	,m_filebuf(new filebuf_type(p))
    {
      this->init(m_filebuf.get());
    }
    basic_ofilestream(const pool_ptr& p,const char* nm,
		      std::ios_base::openmode md,
		      apr_int32_t perms=APR_OS_DEFAULT)
      : ostream_type(NULL)
	,m_filebuf(new filebuf_type(p))
    {
      this->init(m_filebuf.get());
      try{
	if(!this->open(nm,md,perms)){
	  this->setstate(std::ios_base::failbit);
	}
      }OAPR_CATCH()      
    }
    virtual ~basic_ofilestream(){}

    bool is_open()const{return m_filebuf->is_open();}
    ofilestream_type* open(const char* nm, std::ios_base::openmode md,
			  apr_int32_t perms=APR_OS_DEFAULT)
    {
      ofilestream_type* rv=0;
      if(m_filebuf->open(nm,md,perms)){
	rv =this;
      }
      return rv;
    }
    ofilestream_type* close()
    {
      ofilestream_type* rv=0;
      if(m_filebuf->close()){
	rv=this;
      }else{
	this->setstate(std::ios_base::failbit);
      }
      return rv;
    }
  };
};
#endif
