/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2000-2003 Justin Kirby
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 */

#include "oapr/oapr_app.hpp"
#include "oapr/oapr_io_file.hpp"
#include "oapr/oapr_exception.hpp"
#include "oapr/oapr_exc_utils.hpp"
#include <iostream>
#ifdef WIN32
#define SEP "\\"
#define TEST_DIR "c:\\temp"
#else
#define SEP "/"
#define TEST_DIR "/tmp"
#endif
#define TEST_FILENAME1 "oapr-test1"
#define TEST_FILENAME2 "oapr-test2"
#define TEST_FILE1 TEST_DIR SEP TEST_FILENAME1
#define TEST_FILE2 TEST_DIR SEP TEST_FILENAME2
#define TEST_DATA "some random text to write to test file"

void test_create_file()
{
  oapr::pool_mgr::pool_ptr pl=oapr::pool_mgr::instance()->unique();
  oapr::basic_file testCreate(pl);

  std::cout<<"opening file " TEST_FILE1 <<std::endl;
  OAPR_CHECK(testCreate.open(std::string(TEST_FILE1),
		     APR_WRITE | APR_CREATE,
		     APR_OS_DEFAULT));
}

void test_write_file()
{
  oapr::pool_mgr::pool_ptr pl=oapr::pool_mgr::instance()->unique();
  oapr::basic_file testWrite(pl);

  std::string text(TEST_DATA);

  std::cout<<"writing to file " TEST_FILE1 << std::endl;
  OAPR_CHECK(testWrite.open(std::string(TEST_FILE1),APR_WRITE,APR_OS_DEFAULT));
  testWrite.write(text.c_str(),text.size());
}

void test_read_file()
{
  oapr::pool_mgr::pool_ptr pl=oapr::pool_mgr::instance()->unique();
  oapr::basic_file testRead(pl);
  std::string test(TEST_DATA);
  char *buff=new char[test.size()+1];

  OAPR_CHECK(testRead.open(std::string(TEST_FILE1),APR_READ,APR_OS_DEFAULT));
  testRead.read(buff,test.size());
  buff[test.size()+1]='\0';

  std::cout<<buff<<std::endl;
}

void test_copy_file()
{
  std::cout<<"Copying " TEST_FILE1 " to " TEST_FILE2 " .... "<<std::endl;
  OAPR_CHECK(oapr::copy(TEST_FILE1,TEST_FILE2,APR_FILE_SOURCE_PERMS));

}
void test_append_file()
{
  std::cout<<"Appending " TEST_FILE1 " to " TEST_FILE2 " .... "<<std::endl;
  OAPR_CHECK(oapr::append(TEST_FILE1,TEST_FILE2,APR_FILE_SOURCE_PERMS));
}
void test_remove_file()
{
  std::cout<<"Removing " TEST_FILE2 " .... ";
  OAPR_CHECK(oapr::remove(TEST_FILE2));
}
int main(int argc,char*argv[])
{
  try{
    oapr::oapr_app app("testfile");

    test_create_file();
    test_write_file();
    test_read_file();
    test_copy_file();
    test_append_file();
    test_remove_file();
  }catch(const oapr::exception_base& e){
    print_stack(&std::cerr,e);	
  }
  return 0;
}
